/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: ImageEntropy1.cxx,v $
  Language:  C++
  Date:      $Date: 2009-04-07 13:28:38 $
  Version:   $Revision: 1.13 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include "itkScalarImageToHistogramGenerator.h"
#include "itkImage.h"

#include "itkImageFileReader.h"
//#include "itkImageFileWriter.h"
#include "itkStatisticsImageFilter.h"
 
int main( int argc, char * argv [] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing command line arguments" << std::endl;
    std::cerr << "Usage :  ImageHistogram1  inputImageFileName computationType";
    return -1;
    }

// The pixel type and dimension of the image are explicitly declared and then
// used for instantiating the image type.

 // typedef unsigned char       PixelType;
  //const   unsigned int        Dimension = 3;
  //const unsigned int numberOfHistogramBins = atoi( argv[2] );

  typedef float       PixelType;
  const   unsigned int        Dimension = 3;
  const unsigned int numberOfHistogramBins = 256;

  typedef itk::Image< PixelType, Dimension > ImageType;
// Software Guide : EndCodeSnippet

  typedef itk::ImageFileReader< ImageType > ReaderType;

  ReaderType::Pointer reader = ReaderType::New();

  reader->SetFileName( argv[1] );

  try
    {
    reader->Update();
    }
  catch( itk::ExceptionObject & excp )
    {
    std::cerr << "Problem encoutered while reading image file : " << argv[1] << std::endl;
    std::cerr << excp << std::endl;
    return -1;
    }


// The image type is used as template parameter for instantiating the histogram
// generator.
  typedef itk::Statistics::ScalarImageToHistogramGenerator< ImageType >   HistogramGeneratorType;

  HistogramGeneratorType::Pointer histogramGenerator = HistogramGeneratorType::New();
// The parameters of the desired histogram are defined. In particular, the
// number of bins and the marginal scale. For convenience in this example, we
// read the number of bins from the command line arguments. In this way we can
// easily experiment with different values for the number of bins and see how
// that choice affects the computation of the entropy.


  histogramGenerator->SetNumberOfBins( numberOfHistogramBins );
  histogramGenerator->SetMarginalScale( 10.0 );

// We can then connect as input the output image from a reader and trigger the
// histogram computation by invoking the \code{Compute()} method in the
// generator.

  histogramGenerator->SetInput(  reader->GetOutput() );
  
  histogramGenerator->Compute();

// The resulting histogram can be recovered from the generator by using the
// \code{GetOutput()} method. A histogram class can be declared using the
// \code{HistogramType} trait from the generator.

  typedef HistogramGeneratorType::HistogramType  HistogramType;

  const HistogramType * histogram = histogramGenerator->GetOutput();

  const unsigned int histogramSize = histogram->Size();

  //std::cout << "Histogram size " << histogramSize << std::endl;

  for( unsigned int bin=0; bin < histogramSize; bin++ )
    {
   // std::cout << "bin = " << bin << " frequency = ";
    //std::cout << histogram->GetFrequency( bin, 0 ) << std::endl;
    }

// We proceed now to compute the \emph{estimation} of entropy given the
// histogram. The first conceptual jump to be done here is that we assume that
// the histogram, which is the simple count of frequency of occurrence for the
// gray scale values of the image pixels, can be normalized in order to estimate
// the probability density function \textbf{PDF} of the actual statistical
// distribution of pixel values. 
//
//  First we declare an iterator that will visit all the bins in the histogram.
//  Then we obtain the total number of counts using the
//  \code{GetTotalFrequency()} method, and we initialize the entropy variable
//  to zero.  

  HistogramType::ConstIterator itr = histogram->Begin();
  HistogramType::ConstIterator end = histogram->End();
 
  double Sum = histogram->GetTotalFrequency();

  double Entropy = 0.0;
// We start now visiting every bin and estimating the probability of a pixel to
// have a value in the range of that bin. The base 2 logarithm of that
// probability is computed, and then weighted by the probability in order to
// compute the expected amount of information for any given pixel. Note that a
// minimum value is imposed for the probability in order to avoid computing
// logarithms of zeros.
//
//  Note that the $\log{(2)}$ factor is used to convert the natural logarithm
//  in to a logarithm of base 2, and make possible to report the entropy in its
//  natural unit: the bit.
  while( itr != end )
    {
    const double probability = itr.GetFrequency() / Sum;

    if( probability > 0.99 / Sum )
      {
      Entropy += - probability * vcl_log( probability ) / vcl_log( 2.0 );
      }
    ++itr;
    }

// The result of this sum is considered to be our estimation of the image
// entropy. Note that the Entrpy value will change depending on the number of
// histogram bins that we use for computing the histogram. This is particularly
// important when dealing with images whose pixel values have dynamic ranges so
// large that our number of bins will always underestimate the variability of
// the data.

 // std::cout << "Image entropy (in bits) = " << Entropy << std::endl;

// compute mean

  double mean = 0.0;
   for( unsigned int bin=0; bin < histogramSize; bin++ )
    {
	mean=mean+( (histogram->GetFrequency( bin, 0 )*bin)/Sum );
   // std::cout << "bin = " << bin << " frequency = ";
    //std::cout << histogram->GetFrequency( bin, 0 ) << std::endl;
    }
// std::cout << "Image mean = " << mean << std::endl;

// compute Variance

  double variance= 0.0;
   for( unsigned int bin=0; bin < histogramSize; bin++ )
    {
	const double probability = histogram->GetFrequency( bin, 0 ) / Sum;

	variance=variance + probability* pow(bin -mean ,2);
   // std::cout << "bin = " << bin << " frequency = ";
    //std::cout << histogram->GetFrequency( bin, 0 ) << std::endl;
    }
 //std::cout << "Image variance = " << variance << std::endl;
 //std::cout << "Image stdev = " << sqrt(variance) << std::endl;

// compute Skewness
  double skewness= 0.0;
   for( unsigned int bin=0; bin < histogramSize; bin++ )
    {
	const double probability = histogram->GetFrequency( bin, 0 ) / Sum;

	skewness=skewness + probability * pow(bin -mean ,3);
   // std::cout << "bin = " << bin << " frequency = ";
    //std::cout << histogram->GetFrequency( bin, 0 ) << std::endl;
    }

 skewness = (skewness / pow(sqrt(variance),3)) -3;

// std::cout << "Image Skewness = " << skewness << std::endl;

// compute Kurtosis

  double kurtosis= 0.0;
   for( unsigned int bin=0; bin < histogramSize; bin++ )
    {
	const double probability = histogram->GetFrequency( bin, 0 ) / Sum;

	kurtosis=kurtosis + probability * pow(bin -mean ,4);
   // std::cout << "bin = " << bin << " frequency = ";
    //std::cout << histogram->GetFrequency( bin, 0 ) << std::endl;
    }

 kurtosis = (kurtosis / pow(variance,2)) -3;

// std::cout << "Image Kurtosis = " << kurtosis << std::endl;

/*  typedef itk::StatisticsImageFilter<ImageType> StatisticsImageFilterType;

  StatisticsImageFilterType::Pointer statisticsImageFilter = StatisticsImageFilterType::New ();
  statisticsImageFilter->SetInput(reader->GetOutput());
  statisticsImageFilter->Update();
 
  std::cout << "Mean: " << statisticsImageFilter->GetMean() << std::endl;
  std::cout << "Std.: " << statisticsImageFilter->GetSigma() << std::endl;
  std::cout << "Min: " << statisticsImageFilter->GetMinimum() << std::endl;
  std::cout << "Max: " << statisticsImageFilter->GetMaximum() << std::endl;*/

/* std::cout << "Image mean = " << mean << std::endl;
std::cout << "Image stdev = " << sqrt(variance) << std::endl;
 std::cout << "Image Skewness = " << skewness << std::endl;
 std::cout << "Image Kurtosis = " << kurtosis << std::endl;
 std::cout << "Image entropy (in bits) = " << Entropy << std::endl;*/


int type=atoi(argv[2]);
switch(type)
{
  case 0:
  std::cout << mean ;
  break;
 case 1:
  std::cout << sqrt(variance) ;
  break;
 case 2:
 std::cout << skewness ;
break;
 case 3:
 std::cout << kurtosis ;
    break;
  case 4:
 std::cout << Entropy ;
    break;
  case 5:
  std::cout << mean <<"\t"<<sqrt(variance)<<"\t"<< skewness <<"\t"<< kurtosis <<"\t"<< Entropy <<std::endl;

  // summary of the different values 	
/*  double w1,w2,w3,w4,statsSummary;
  w1=0.0;		
  w2=0.0;		
  w3=0.0;		
  w4=0.0;		
  statsSummary=0.0;		
  std::cout << Entropy ;*/
    break;

  default:
 exit(-1); 
}

  return 0;
  
}


