#include <brainio.h>

BrainIO::BrainIO()
{
}

IntensityImage BrainIO::ReadIntensityImage(FileNameType fileName) {
    // Init
    IntensityReaderType::Pointer intreader = IntensityReaderType::New();;
    IntensityImageType::Pointer intimage = NULL;

    if (fileName.compare("")!=0) {
        intreader->SetFileName( fileName );
        // Reading the image.
        try {
            intreader->Update();
            intimage = intreader->GetOutput();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
    return(intimage);
}

ProbabilityImage BrainIO::ReadProbabilityImage(FileNameType fileName) {
    // Init
    ProbabilityReaderType::Pointer freader = ProbabilityReaderType::New();
    ProbabilityImageType::Pointer fimage = NULL;

    if (fileName.compare("")!=0) {
        freader->SetFileName( fileName );
        // Reading the image.
        try {
            freader->Update();
            fimage = freader->GetOutput();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
    return(fimage);
}

ResultsImage BrainIO::ReadResultsImage(FileNameType fileName) {
    // Init
    ResultsReaderType::Pointer rreader = ResultsReaderType::New();
    ResultsImageType::Pointer rimage = NULL;

    if (fileName.compare("")!=0) {
        rreader->SetFileName( fileName );
        // Reading the image.
        try {
            rreader->Update();
            rimage = rreader->GetOutput();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
    return(rimage);
}

MaskImage BrainIO::ReadMaskImage(FileNameType fileName) {
    // Init
    MaskReaderType::Pointer breader = MaskReaderType::New();
    MaskImageType::Pointer bimage = NULL;

    if (fileName.compare("")!=0) {
        breader->SetFileName( fileName );
        // Reading the image.
        try {
            breader->Update();
            bimage = breader->GetOutput();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
    return(bimage);
}

void BrainIO::WriteIntensityImage(FileNameType fileName,IntensityImage intimage) {
    // Init
    IntensityWriterType::Pointer intwriter = IntensityWriterType::New();

    if (fileName.compare("")!=0) {
        intwriter->SetFileName( fileName );
        intwriter->SetInput( intimage );
        // Reading the image.
        try {
            intwriter->Update();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
}

void BrainIO::WriteResultsImage(FileNameType fileName,ResultsImage resimage) {
    // Init
    ResultsWriterType::Pointer reswriter = ResultsWriterType::New();

    if (fileName.compare("")!=0) {
        reswriter->SetFileName( fileName );
        reswriter->SetInput( resimage );
        // Reading the image.
        try {
            reswriter->Update();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
}

void BrainIO::WriteProbabilityImage(FileNameType fileName,ProbabilityImage fimage) {
    // Init
    ProbabilityWriterType::Pointer fwriter = ProbabilityWriterType::New();

    if (fileName.compare("")!=0) {
        fwriter->SetFileName( fileName );
        fwriter->SetInput( fimage );
        // Reading the image.
        try {
            fwriter->Update();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
}

void BrainIO::WriteMaskImage(FileNameType fileName,MaskImage bimage) {
    // Init
    MaskWriterType::Pointer mwriter = MaskWriterType::New();

    if (fileName.compare("")!=0) {
        mwriter->SetFileName( fileName );
        mwriter->SetInput( bimage );
        // Reading the image.
        try {
            mwriter->Update();
        }
        catch (itk::ExceptionObject & e) {
            std::cerr << "exception in file reader " << std::endl;
            std::cerr << e.GetDescription() << std::endl;
            std::cerr << e.GetLocation() << std::endl;
        }
    }
}
