#ifndef BRAINIO_H
#define BRAINIO_H

#include <itkImageFileReader.h>
#include <itkImageFileWriter.h>

#include <imagedefinitions.h>

class BrainIO
{

    // Intensity Images (T1 basically)
    typedef itk::ImageFileReader< IntensityImageType > IntensityReaderType;
    typedef itk::ImageFileWriter< IntensityImageType > IntensityWriterType;

    // Probability atlases
    typedef itk::ImageFileReader< ProbabilityImageType > ProbabilityReaderType;
    typedef itk::ImageFileWriter< ProbabilityImageType > ProbabilityWriterType;

    // Segmentation results
    typedef itk::ImageFileReader< ResultsImageType > ResultsReaderType;
    typedef itk::ImageFileWriter< ResultsImageType > ResultsWriterType;

    // Masks
    typedef itk::ImageFileReader< MaskImageType > MaskReaderType;
    typedef itk::ImageFileWriter< MaskImageType > MaskWriterType;

public:
    BrainIO();

    // Read
    ProbabilityImage ReadProbabilityImage(FileNameType fileName);
    IntensityImage ReadIntensityImage(FileNameType fileName);
    ResultsImage ReadResultsImage(FileNameType fileName);
    MaskImage ReadMaskImage(FileNameType fileName);

    // Write
    void WriteProbabilityImage(FileNameType fileName,ProbabilityImage fimage);
    void WriteIntensityImage(FileNameType fileName,IntensityImage intimage);
    void WriteResultsImage(FileNameType fileName,ResultsImage resimage);
    void WriteMaskImage(FileNameType fileName,MaskImage bimage);
};

#endif // BRAINIO_H
