#ifndef BRAINREGISTRATION_H
#define BRAINREGISTRATION_H

#include <vector>
#include <cstdlib>


#include <imagedefinitions.h>
#include <registrationlevelupdate.h>

#include "./src/definicionsTipus.h"

class BrainRegistration
{

public:
    BrainRegistration();

    // Setters
    void SetFixed(ProbabilityImage fix);
    void SetMoving(ProbabilityImage mov);
    void SetMask(MaskImage mask);
    void SetAnatomic(ResultsImage mask);
    void AddAtlas(ProbabilityImage atlas);
    void SetAtlases(std::vector<ProbabilityImage> atlases);
    void ClearAtlases();
    // Getters
    ProbabilityImage GetFixed();
    ProbabilityImage GetMoving();
    MaskImage GetMask();
    ResultsImage GetAnatomic();
    std::vector<ProbabilityImage> GetAtlases();

    // Metric options
    void UseMutualInformationMetric();
    void UseMattesMutualInformationMetric();
    void UseMutualInformationHistogramMetric();
    void UseNormalizedMutualInformationMetric();
    void UseGradientDifferenceMetric();
    void UseKullbackLeiblerMetric();
    void UseKappaStatisticMetric();
    void UseCorrelationMetric();
    void UseNormalizedCorrelationMetric();
    void UseMeanSquaresMetric();
    void UseMeanSquaresHistogramMetric();
    void UseMeanReciprocalSquaresMetric();

    // Registration methods
    void RigidRegistration(int levels = 2, int steps = 50);
    void AffineRegistration(int levels = 2, int steps = 50);
    void BSplineMultiRegistration(int levels = 2, unsigned int ngridnodesdim = 5, int steps = 200);
    void BSplineRegistration(unsigned int ngridnodesdim = 10, int steps = 200);
    void DemonsRegistration();

    static ResultsImage UpSampling(ResultsImage label, int scale=2);

private:

    void ResampleBSpline();
    SignedImageType::Pointer DownsampleImage(SignedImageType::Pointer in);

    // Registration done flags
    bool rigid;
    bool affine;
    bool bspline;

    // Registration parameters
    int rigidlevels;
    int affinelevels;
    double bsplinenodes;

    // Image based pointers
    ProbabilityImage fiximg;
    IntensityRegionType fixreg;
    IntensitySizeType fixsize;
    OriginType origin;
    SpacingType spacing;
    IntensityDirectionType griddir;
    ProbabilityImage movimg;
    MaskImage maskimg;
    ResultsImage anaimg;
    IntensityRegionType movreg;
    IntensitySizeType movsize;
    OriginType movorigin;
    SpacingType movspacing;
    std::vector<ProbabilityImage> movatlas;

    // Region based pointers
    RegionType::SizeType gridsizeimg;
    RegionType::SizeType gridbordersize;
    RegionType::SizeType totalgridsize;

    // Transform pointers
    IdentityTransformType::Pointer idtransf;
    RigidTransformType::Pointer rigidtransf;
    AffineTransformType::Pointer affinetransf;
    DeformableTransformType::Pointer bsplinetransf;
    RegionType bsplinereg;
    TransformInitializerType::Pointer initializer;

    // Metric pointers
    MIMetricType::Pointer mi_metric;
    MattesMIMetricType::Pointer mattesmi_metric;
    MIHistogramMetricType::Pointer mihist_metric;
    NMIMetricType::Pointer nmi_metric;
    KLHistogramMetricType::Pointer kl_metric;
    GDMetricType::Pointer gd_metric;
    KappaMetricType::Pointer kappa_metric;
    CorrelationMetricType::Pointer corr_metric;
    NormalizedCorrelationMetricType::Pointer ncorr_metric;
    MSMetricType::Pointer ms_metric;
    MSHistogramMetricType::Pointer mshist_metric;
    MSReciprocalMetricType::Pointer msr_metric;


    // Optimizer pointers
    OptimizerType::Pointer optimizer;
    VersorOptimizerType::Pointer versor_optimizer;

    // Interpolator pointers
    linInterpolatorType::Pointer interpolator;
    BSplineImageInterpolatorType::Pointer bintinterpolator;
    BSplineAtlasInterpolatorType::Pointer bprinterpolator;
    BSplineInterpolatorType::Pointer binterpolator;
    NNInterpolatorType::Pointer ninterpolator;
    NNAnatomicInterpolatorType::Pointer nanainterpolator;

    // Registration methods
    mRegistrationType::Pointer registration;
    MultiResolutionRegistrationType::Pointer multires_registration;

    // Resampling pointers
    ResampleImageFilterType::Pointer imresample;
    ResampleAtlasFilterType::Pointer atlasresample;
    ResampleMaskFilterType::Pointer maskresample;
    ResampleResultsFilterType::Pointer anatomicresample;
    MultiResolutionPyramidType::Pointer multires_pyramidfix;
    MultiResolutionPyramidType::Pointer multires_pyramidmov;

    // Casting pointers
    IntensityToSignedCastType::Pointer fixint2signed;
    IntensityToSignedCastType::Pointer movint2signed;
    IntensityToSignedCastType::Pointer transfint2signed;

    // Command pointer
    ObserverType::Pointer observer;

    // PreProcessing pointers
    MatchingFilterType::Pointer matcher;

    // Others
    MinimumMaximumImageCalculatorType::Pointer minmax;
};

#endif // BRAINREGISTRATION_H
