/*=========================================================================

  Inspired in (mainly the same!) :    WarpImageFilter1.cxx
 
=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif

#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

#include "itkWarpImageFilter.h"
#include "itkLinearInterpolateImageFunction.h"
#include "itkNearestNeighborInterpolateImageFunction.h"

int main( int argc, char * argv[] )
{
  if( argc < 4 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  inputDeformationField  outputImageFile" << std::endl;
    return EXIT_FAILURE;
    }

  const     unsigned int   Dimension = 3;

  typedef   float VectorComponentType;

  typedef   itk::Vector< VectorComponentType, Dimension > VectorPixelType;
  typedef   itk::Image< VectorPixelType,  Dimension >   DeformationFieldType;
  

//  typedef   unsigned char  PixelType;
  typedef   float  PixelType;
  typedef   itk::Image< PixelType,  Dimension >   ImageType;

  typedef   itk::ImageFileReader< ImageType >  ReaderType;
  typedef   itk::ImageFileWriter< ImageType >  WriterType;
  
  typedef   itk::ImageFileReader< DeformationFieldType >  FieldReaderType;

  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();

  reader->SetFileName( argv[1] );
  writer->SetFileName( argv[3] );


  FieldReaderType::Pointer fieldReader = FieldReaderType::New();
  fieldReader->SetFileName( argv[2] );
  fieldReader->Update();

  DeformationFieldType::ConstPointer deformationField = fieldReader->GetOutput();

  typedef itk::WarpImageFilter< ImageType, ImageType, DeformationFieldType  >  FilterType;

  FilterType::Pointer filter = FilterType::New();

  //typedef itk::LinearInterpolateImageFunction< ImageType, double >  InterpolatorType;
  typedef itk::NearestNeighborInterpolateImageFunction< ImageType, double >  InterpolatorType;

  InterpolatorType::Pointer interpolator = InterpolatorType::New();

  filter->SetInterpolator( interpolator );

  filter->SetOutputSpacing( deformationField->GetSpacing() );
  filter->SetOutputOrigin(  deformationField->GetOrigin() );
  filter->SetOutputDirection(  deformationField->GetDirection() );

  filter->SetDeformationField( deformationField );

  filter->SetInput( reader->GetOutput() );
  writer->SetInput( filter->GetOutput() );

 
  try
    {
    writer->Update();
    }
  catch( itk::ExceptionObject & excp )
    {
    std::cerr << "Exception thrown " << std::endl;
    std::cerr << excp << std::endl;
    }

  return EXIT_SUCCESS;

}

