#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#include<iostream>
#include <itkVector.h>
#include <itkImage.h>
#include "itkImageFileReader.h" 
#include <itkDisplacementFieldJacobianDeterminantFilter.h>

using namespace std;
 
int main( int argc, char * argv [] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing command line arguments" << std::endl;
    std::cerr << "Usage :  DeformationFieldFile type of computation";
    return -1;
    }


  typedef float       PixelType;
  const   unsigned int        Dimension = 3;

     typedef itk::Vector< PixelType, Dimension  > VectorPixelType;
     typedef itk::Image< VectorPixelType, Dimension > DeformationFieldType;
     typedef itk::ImageFileReader< DeformationFieldType > ReaderType;

     ReaderType::Pointer reader = ReaderType::New();

     reader->SetFileName(argv[1]);

   reader->Update();

     DeformationFieldType::Pointer deffield = reader->GetOutput();

 
  VectorPixelType VectorValue;

  DeformationFieldType::SizeType size = deffield->GetLargestPossibleRegion().GetSize();

 DeformationFieldType::IndexType vectorIndex;

// First computations on norm 
double norm;
long numPixels=size[0]*size[1]*size[2];
double maxNorm=0;
double minNorm=9999999;
long double averageNorm=0;


  for(unsigned int x = 0; x < size[0]; x++)
  {
    for(unsigned int y = 0; y < size[1]; y++)
    {
	for(unsigned int z = 0; z < size[2]; z++)
	    {
		vectorIndex[0] = x; // x position
		vectorIndex[1] = y; // y position
		vectorIndex[2] = z; // y position

		// compute current norm
		norm = sqrt(pow(VectorValue[0],2) + pow(VectorValue[1],2) + pow(VectorValue[2],2));

		averageNorm=averageNorm+(norm/numPixels);

		if(maxNorm<norm) maxNorm=norm;
		if( (minNorm>norm)&&(norm>0.000000001)) minNorm=norm;


		         //std::cout<<"pixel "<<vectorIndex<<endl;
	 

		          VectorValue = deffield->GetPixel( vectorIndex);

		          //VectorValue[2] = 0.0; // z component

		          deffield->SetPixel(vectorIndex, VectorValue);
	}
     }
  }

averageNorm=averageNorm;


//std::cout<<"Norm average: "<<averageNorm<<" max: "<<maxNorm<<" min: "<<minNorm<<" number of pixels "<<numPixels<<std::endl;



// now compute jacobian


typedef itk::DisplacementFieldJacobianDeterminantFilter<DeformationFieldType, PixelType> JacobianFilterType;
JacobianFilterType::Pointer m_JacobianFilter;
m_JacobianFilter = JacobianFilterType::New();
m_JacobianFilter->SetUseImageSpacing( true );
m_JacobianFilter->ReleaseDataFlagOn();

m_JacobianFilter->SetInput( deffield );
m_JacobianFilter->UpdateLargestPossibleRegion();
      
const unsigned int numPix = m_JacobianFilter-> GetOutput()->GetLargestPossibleRegion().GetNumberOfPixels();
      
PixelType* pix_start = m_JacobianFilter->GetOutput()->GetBufferPointer();
PixelType* pix_end = pix_start + numPix;
      
PixelType* jac_ptr;

// Get percentage of det(Jac) below 0
unsigned int jacBelowZero(0u);
for (jac_ptr=pix_start; jac_ptr!=pix_end; ++jac_ptr)
{
if ( *jac_ptr<=0.0 ) ++jacBelowZero;
}
const double jacBelowZeroPrc = static_cast<double>(jacBelowZero)
 / static_cast<double>(numPix);

// Get min an max jac
const double minJac = *(std::min_element (pix_start, pix_end));
const double maxJac = *(std::max_element (pix_start, pix_end));

// Get some quantiles
// We don't need the jacobian image
// we can modify/sort it in place
jac_ptr = pix_start + static_cast<unsigned int>(0.002*numPix);
std::nth_element(pix_start, jac_ptr, pix_end);
const double Q002 = *jac_ptr;

jac_ptr = pix_start + static_cast<unsigned int>(0.01*numPix);
std::nth_element(pix_start, jac_ptr, pix_end);
const double Q01 = *jac_ptr;

jac_ptr = pix_start + static_cast<unsigned int>(0.99*numPix);
std::nth_element(pix_start, jac_ptr, pix_end);
const double Q99 = *jac_ptr;

jac_ptr = pix_start + static_cast<unsigned int>(0.998*numPix);
std::nth_element(pix_start, jac_ptr, pix_end);
const double Q998 = *jac_ptr;

//std::cout<<"max|Jac| "<<maxJac<<" - " <<"min|Jac| "<<minJac<<" - "<<"ratio(|Jac|<=0) "<<jacBelowZeroPrc<<std::endl;  

//std::cout<<", "<<Q002<<", "<<Q01<<", "<<Q99<<", "<<Q998<<std::endl;

int type=atoi(argv[2]);
switch(type)
{
  case 0:
 	std::cout<<minNorm;
  break;
  case 1:
 	std::cout<<averageNorm;
  break;
  case 2:
 	std::cout<<maxNorm;
  break;
  case 3:
 	std::cout<<minJac;
  break;
  case 4:
 	std::cout<<maxJac;
  break;
  case 5:
 	std::cout<<jacBelowZeroPrc;
  break;
  case 6:
 	std::cout<<minNorm<<"\t"<<averageNorm<<"\t"<<maxNorm<<"\t"<<maxJac<<"\t"<<minJac<<"\t"<<jacBelowZeroPrc<<std::endl;
  break;
  default:
 exit(-1); 
}
  return 0;
  
}












//KIIIIIIIIIIIIIIIIIIIIIIIIIIII
/*typedef itk::Vector< float, 3  > VectorPixelType;
     typedef itk::Image< VectorPixelType, 3 > DeformationFieldType;
     typedef itk::ImageFileReader< DeformationFieldType > ReaderType;

     ReaderType::Pointer reader = ReaderType::New();

     reader->SetFileName("deformationField.mhd");

     reader->Update();

     DeformationFieldType::ConstPointer field =
                                     reader->GetOutput();










DeformationFieldType::PixelType VectorValue;

  DeformationFieldType::SizeType size = deformationField->GetLargestPossibleRegion().GetSize();

 DeformationFieldType::IndexType vectorIndex;

                

  for(int y = 0; y < size[1]; y++)

  for(int x = 0; x < size[0]; x++)

  {

                  vectorIndex[0] = x; // x position

                vectorIndex[1] = y; // y position

                

 

                  VectorValue = deformationField->GePixelType( vectorIndex);

                  VectorValue[2] = 0.0; // z component

                  deformationField->SePixelType(vectorIndex, VectorValue);

  }
*/






