#ifndef IMAGEDEFINITIONS_H
#define IMAGEDEFINITIONS_H
#include <itkImage.h>
#include <itkImageRegionIterator.h>
#include <itkNeighborhoodIterator.h>

#include <itkVector.h>
#include <itkMatrix.h>
#include <itkVariableSizeMatrix.h>
#include <itkCompose2DVectorImageFilter.h>
#include <itkCompose3DVectorImageFilter.h>
#include <itkImageToVectorImageFilter.h>

#include <itkVTKImageExport.h>
#include <itkTimeProbesCollectorBase.h>
#include <itkMemoryProbesCollectorBase.h>

#ifdef ITK_USE_REVIEW
#define itkProbesCreate()  \
  itk::TimeProbesCollectorBase chronometer; \
  itk::MemoryProbesCollectorBase memorymeter
#define itkProbesStart( text ) memorymeter.Start( text ); chronometer.Start( text )
#define itkProbesStop( text )  chronometer.Stop( text ); memorymeter.Stop( text  )
#define itkProbesReport( stream )  chronometer.Report( stream ); memorymeter.Report( stream  )
#else
#define itkProbesCreate()  \
  itk::TimeProbesCollectorBase chronometer
#define itkProbesStart( text ) chronometer.Start( text )
#define itkProbesStop( text )  chronometer.Stop( text )
#define itkProbesReport( stream )  chronometer.Report( stream )
#endif


typedef unsigned short IntensityPixelType;
typedef itk::Image< IntensityPixelType, 3 > IntensityImageType;
typedef IntensityImageType::Pointer IntensityImage;
typedef IntensityImageType::RegionType IntensityRegionType;
typedef IntensityImageType::SizeType IntensitySizeType;
typedef IntensityImageType::DirectionType IntensityDirectionType;
typedef itk::ImageRegionIterator<IntensityImageType> IntensityIterator;
typedef itk::NeighborhoodIterator<IntensityImageType> IntensityNeighborhoodIterator;



//typedef double ProbabilityPixelType;
typedef float ProbabilityPixelType;
typedef itk::Image< ProbabilityPixelType, 3 > ProbabilityImageType;
typedef ProbabilityImageType::Pointer ProbabilityImage;
typedef ProbabilityImageType::RegionType ProbabilityRegionType;
typedef ProbabilityImageType::SizeType ProbabilitySizeType;
typedef ProbabilityImageType::DirectionType ProbabilityDirectionType;
typedef itk::ImageRegionIterator<ProbabilityImageType> ProbabilityIterator;
typedef itk::NeighborhoodIterator<ProbabilityImageType> ProbabilityNeighborhoodIterator;

typedef float SignedIntensityPixelType;
typedef itk::Image< SignedIntensityPixelType, 3 > SignedImageType;
typedef SignedImageType::Pointer SignedImage;
typedef SignedImageType::RegionType SignedRegionType;
typedef SignedImageType::SizeType SignedSizeType;
typedef SignedImageType::DirectionType SignedDirectionType;
typedef SignedImageType::SizeType::SizeValueType SignedSizeValueType;

typedef unsigned char ResultsPixelType;
typedef itk::Image< ResultsPixelType, 3 > ResultsImageType;
typedef ResultsImageType::Pointer ResultsImage;
typedef ResultsImageType::RegionType ResultsRegionType;
typedef ResultsImageType::SizeType ResultsSizeType;
typedef ResultsImageType::DirectionType ResultsDirectionType;
typedef itk::ImageRegionIterator<ResultsImageType> ResultsIterator;
typedef itk::NeighborhoodIterator<ResultsImageType> ResultsNeighborhoodIterator;

typedef bool MaskPixelType;
typedef itk::Image< MaskPixelType, 3 > MaskImageType;
typedef MaskImageType::Pointer MaskImage;
typedef itk::ImageRegionIterator<MaskImageType> MaskIterator;
typedef itk::NeighborhoodIterator<MaskImageType> MaskNeighborhoodIterator;

typedef std::string FileNameType;

enum { MI_MATTES, MI, MI_HIST, NMI, CORR, NORM_CORR, GRADIENT_DIFF, KULLBACK_LEIBLER, KAPPA, MEAN_SQUARES, MEAN_SQUARES_HIST, MEAN_SQUARES_RECIPROCAL };

template <class T> const T& min ( const T& a, const T& b ) {
  return (a<b)?a:b;     // or: return comp(a,b)?a:b; for the comp version
}
template <class T, class Tt> const T& min ( const T& a, const Tt& b ) {
  return (a<b)?a:b;     // or: return comp(a,b)?a:b; for the comp version
}

#endif // IMAGEDEFINITIONS_H
