#include "itkTransformFileReader.h"
#include "itkTransformFileWriter.h"

#include "itkTransformFactory.h"
//#include "./src/definicionsTipus.h"

// Read Affine transform previously wrote by writeTransformAffine method
AffineTransformType::Pointer readTransformAffine(std::string filename){
  
    // Some transforms (like the BSpline transform) might not be registered 
    // with the factory so we add them manually. 
    itk::TransformFactory<AffineTransformType>::RegisterTransform();  
    typedef itk::TransformFileReader TransformReaderType;
    TransformReaderType::Pointer transformReader = TransformReaderType::New();
    transformReader->SetFileName( filename.c_str());
    try{
    transformReader->Update();    
    }
    catch( itk::ExceptionObject & excp ){
    std::cerr << "Error while reading affine transform file" << std::endl;
    std::cerr << excp << std::endl;
    }
    
    TransformReaderType::TransformListType * transformList= transformReader->GetTransformList();
    TransformReaderType::TransformListType::const_iterator it = transformList->begin();
    //std::cout << "Number of transforms = " << transformList->size() << std::endl;

    AffineTransformType::Pointer affineTransform = static_cast<AffineTransformType*>((*it).GetPointer());
    //AffineTransformType::Pointer affinetransform = dynamic_cast< AffineTransformType * >( affine_read.GetPointer() );

    return affineTransform;

}

// Read BSpline transform previously wrote by writeTransformBSpline method
BsplineTransformType::Pointer readTransformBSpline(std::string filename){

  // Some transforms (like the BSpline transform) might not be registered 
  // with the factory so we add them manually. 
  itk::TransformFactory<BsplineTransformType>::RegisterTransform();  
  
  typedef itk::TransformFileReader TransformReaderType;  
  TransformReaderType::Pointer transformReader = TransformReaderType::New();
  transformReader->SetFileName( filename.c_str());
  try{
    transformReader->Update();    
  }
  catch( itk::ExceptionObject & excp ){
    std::cerr << "Error while reading BSpline transform file" << std::endl;
    std::cerr << excp << std::endl;
  }
    
  TransformReaderType::TransformListType * transformList= transformReader->GetTransformList();
  TransformReaderType::TransformListType::const_iterator it = transformList->begin();
  //std::cout << "Number of transforms = " << transformList->size() << std::endl;

  BsplineTransformType::Pointer bsplineTransform = static_cast<BsplineTransformType*>((*it).GetPointer());
  
  return bsplineTransform;

}

// Read Rigid transform previously written by writeTransformRigid method
RigidTransformType::Pointer readTransformRigid(std::string filename){
  
    // Some transforms (like the BSpline transform) might not be registered 
    // with the factory so we add them manually. 
    itk::TransformFactory<RigidTransformType>::RegisterTransform();  
    typedef itk::TransformFileReader TransformReaderType;
    TransformReaderType::Pointer transformReader = TransformReaderType::New();
    transformReader->SetFileName( filename.c_str());
    try{
    transformReader->Update();    
    }
    catch( itk::ExceptionObject & excp ){
    std::cerr << "Error while reading Rigid transform file" << std::endl;
    std::cerr << excp << std::endl;
    }
    
    TransformReaderType::TransformListType * transformList= transformReader->GetTransformList();
    TransformReaderType::TransformListType::const_iterator it = transformList->begin();
    //std::cout << "Number of transforms = " << transformList->size() << std::endl;

    RigidTransformType::Pointer rigidTransform = static_cast<RigidTransformType*>((*it).GetPointer());
    //RigidTransformType::Pointer rigidtransform = dynamic_cast< RigidTransformType * >( rigid_read.GetPointer() );

    return rigidTransform;

}


// Write Affine transform previously computed by Affine registration method
int writeTransformAffine(AffineTransformType::Pointer transform, std::string filename){

    typedef itk::TransformFileWriter TransformWriterTypeTransf;
    TransformWriterTypeTransf::Pointer writerTransform = TransformWriterTypeTransf::New();
    writerTransform->SetInput(transform);
    writerTransform->SetFileName(filename.c_str());
    try{
        writerTransform->Update();
    }
    catch( itk::ExceptionObject & excp )
    {
        std::cerr << "Exception thrown writeTransformAffine" << std::endl;
        std::cerr << excp << std::endl;
        return EXIT_FAILURE;
    }    
    return EXIT_SUCCESS;
}

// Write BSpline transform previously computed by BSpline registration method
int writeTransformBSpline(BsplineTransformType::Pointer transform, std::string filename){

    typedef itk::TransformFileWriter TransformWriterTypeTransf;
    TransformWriterTypeTransf::Pointer writerTransform = TransformWriterTypeTransf::New();
    writerTransform->SetInput(transform);
    writerTransform->SetFileName(filename.c_str());
    try{
        writerTransform->Update();
    }
    catch( itk::ExceptionObject & excp )
    {
        std::cerr << "Exception thrown writeTransformBspline" << std::endl;
        std::cerr << excp << std::endl;
        return EXIT_FAILURE;
    }    
    return EXIT_SUCCESS;
}


// Write Rigid transform previously computed by Rigid registration method
int writeTransformRigid(RigidTransformType::Pointer transform, std::string filename){

    typedef itk::TransformFileWriter TransformWriterTypeTransf;
    TransformWriterTypeTransf::Pointer writerTransform = TransformWriterTypeTransf::New();
    writerTransform->SetInput(transform);
    writerTransform->SetFileName(filename.c_str());
    try{
        writerTransform->Update();
    }
    catch( itk::ExceptionObject & excp )
    {
        std::cerr << "Exception thrown writeTransformRigid" << std::endl;
        std::cerr << excp << std::endl;
        return EXIT_FAILURE;
    }    
    return EXIT_SUCCESS;
}



//Apply Affine transform previously computed by a registration method to image
OutputImageTypeTransf::Pointer applyTransformAffine(FixedImageType::Pointer fixedImage,  MovingImageType::Pointer movingImage, AffineTransformType::Pointer transform){

  ResampleFilterType::Pointer resampler = ResampleFilterType::New();  
  //ResampleImageFilterType::Pointer resampler = ResampleImageFilterType::New();  
  resampler->SetTransform( transform );
  resampler->ReleaseDataFlagOn(); //OJU AFEGIT
  resampler->SetInput( movingImage );
  resampler->SetSize(    fixedImage->GetLargestPossibleRegion().GetSize() );
  resampler->SetOutputOrigin(  fixedImage->GetOrigin() );
  resampler->SetOutputSpacing( fixedImage->GetSpacing() );
  resampler->SetOutputDirection( fixedImage->GetDirection() );
 
  //RECORDEM!!! typedef itk::NearestNeighborInterpolateImageFunction< FixedImageType, double >  filterInterpolatorType;
  filterInterpolatorType::Pointer interpolatorNN = filterInterpolatorType::New();
  resampler->SetInterpolator(interpolatorNN);	
 
  // This value is set to zero in order to make easier to perform
  // regression testing in this example. However, for didactic 
  // exercise it will be better to set it to a medium gray value
  // such as 100 or 128.
  resampler->SetDefaultPixelValue( 0 );
  
  try{
    resampler->Update();
  }
  catch( itk::ExceptionObject & err ){
    std::cerr << "ExceptionObject caught applyTransformAffine" << std::endl;
    std::cerr << err << std::endl;
  } 
  
  CastFilterTypeTransf::Pointer c = CastFilterTypeTransf::New();
  c->SetInput(resampler->GetOutput());  
  c->Update();
  
  return c->GetOutput();
}

//Apply Rigid transform previously computed by a registration method to image
OutputImageTypeTransf::Pointer applyTransformRigid(FixedImageType::Pointer fixedImage,  MovingImageType::Pointer movingImage, RigidTransformType::Pointer transform){

  ResampleFilterType::Pointer resampler = ResampleFilterType::New();  
  //ResampleImageFilterType::Pointer resampler = ResampleImageFilterType::New();  
  resampler->SetTransform( transform );
  resampler->ReleaseDataFlagOn(); //OJU AFEGIT
  resampler->SetInput( movingImage );
  resampler->SetSize(    fixedImage->GetLargestPossibleRegion().GetSize() );
  resampler->SetOutputOrigin(  fixedImage->GetOrigin() );
  resampler->SetOutputSpacing( fixedImage->GetSpacing() );
  resampler->SetOutputDirection( fixedImage->GetDirection() );
 
  //RECORDEM!!! typedef itk::NearestNeighborInterpolateImageFunction< FixedImageType, double >  filterInterpolatorType;
  filterInterpolatorType::Pointer interpolatorNN = filterInterpolatorType::New();
  resampler->SetInterpolator(interpolatorNN);	

  // This value is set to zero in order to make easier to perform
  // regression testing in this example. However, for didactic 
  // exercise it will be better to set it to a medium gray value
  // such as 100 or 128.
  resampler->SetDefaultPixelValue( 0 );
  
  try{

    resampler->Update();
  }
  catch( itk::ExceptionObject & err ){
    std::cerr << "ExceptionObject caught applyTransformRigid" << std::endl;
    std::cerr << err << std::endl;
  } 
  
  CastFilterTypeTransf::Pointer c = CastFilterTypeTransf::New();
  c->SetInput(resampler->GetOutput());  
  c->Update();
  
  return c->GetOutput();
}

//Apply BSpline transform previously computed by a registration method to image
OutputImageTypeTransf::Pointer applyTransformBSpline(FixedImageType::Pointer fixedImage,  MovingImageType::Pointer movingImage, BsplineTransformType::Pointer transform){

  ResampleFilterType::Pointer resampler = ResampleFilterType::New();  
  //ResampleImageFilterType::Pointer resampler = ResampleImageFilterType::New();  
  resampler->SetTransform( transform );
  resampler->ReleaseDataFlagOn(); //OJU AFEGIT
  resampler->SetInput( movingImage );
  resampler->SetSize(    fixedImage->GetLargestPossibleRegion().GetSize() );
  resampler->SetOutputOrigin(  fixedImage->GetOrigin() );
  resampler->SetOutputSpacing( fixedImage->GetSpacing() );
  resampler->SetOutputDirection( fixedImage->GetDirection() );

  //RECORDEM!!! typedef itk::NearestNeighborInterpolateImageFunction< FixedImageType, double >  filterInterpolatorType;
  filterInterpolatorType::Pointer interpolatorNN = filterInterpolatorType::New();
  resampler->SetInterpolator(interpolatorNN);	
  
  // This value is set to zero in order to make easier to perform
  // regression testing in this example. However, for didactic 
  // exercise it will be better to set it to a medium gray value
  // such as 100 or 128.
  resampler->SetDefaultPixelValue( 0 );
  
  try{
    resampler->Update();
  }
  catch( itk::ExceptionObject & err ){
    std::cerr << "ExceptionObject caught applyTransformBSpline" << std::endl;
    std::cerr << err << std::endl;
  } 
  
  CastFilterTypeTransf::Pointer c = CastFilterTypeTransf::New();
  c->SetInput(resampler->GetOutput());  
  c->Update();
  
  return c->GetOutput();
}



int writeImage(OutputImageTypeTransf::Pointer outputImage, std::string filename){
    
  WriterTypeTransf::Pointer outputImageWriter = WriterTypeTransf::New();    
  outputImageWriter->SetFileName( filename );
  outputImageWriter->SetInput( outputImage );
  try{
    outputImageWriter->Update();
  }
  catch(itk::ExceptionObject & excp){
    std::cerr << "Exception thrown writing image!!!" << std::endl;
    std::cerr << excp << std::endl;
    return EXIT_FAILURE;
  }
  return EXIT_SUCCESS;
}

FixedImageType::Pointer readFixedImage(std::string filename){
  FixedImageReaderType::Pointer fixedImageReader = FixedImageReaderType::New();
  fixedImageReader->SetFileName( filename );
  fixedImageReader->Update();  
  return fixedImageReader->GetOutput();    
}
MovingImageType::Pointer readMovingImage(std::string filename){
  MovingImageReaderType::Pointer movingImageReader = MovingImageReaderType::New();
  movingImageReader->SetFileName( filename );
  movingImageReader->Update();  
  return movingImageReader->GetOutput();
}

