#include <iostream>
#include <exception>
#include <cstdlib>
#include <cstdio>

#include <brainio.h>
#include <brainregistration.h>
//#include "./src/definicionsTipus.h"
#include "./src/funcionsAuxiliars.cxx"


using namespace std;

int main(int argc, char **argv)
{

    /* Init */
    int argn = 1;
    if ( (argc<2) || ((atoi(argv[1])==1) && (argc<6)) || ((atoi(argv[1])==2) && (argc<8)) || ((atoi(argv[1])==3) && (argc<11))) {
        cerr << argv[0] << " <method (1: Rigid multiresolution)> <fixed_img> <moving_img> <levels_rigid> <iterations_rigid>" << endl << endl;
        cerr << argv[0] << " <method (2: Affine multiresolution)> <fixed_img> <moving_img> <levels_rigid> <iterations_rigid> <levels_affine> <iterations_affine>" << endl << endl;
        cerr << argv[0] << " <method (3: B-Splines multiresolution)> <fixed_img> <moving_img> <levels_rigid> <iterations_rigid> <levels_affine> <iterations_affine> <nodes_bspline> <levels_bspline> <iterations_bspline>" << endl << endl;
        return EXIT_FAILURE;
    }

    // Variables
    int levelsRigid, levelsAffine, levelsBspline;
    int iterationsRigid, iterationsAffine, iterationsBspline;
    int nodesBspline;
    int method;
    FileNameType fixedImageName, movingImageName;
    char * outputImageName = new char[200];

    // Classes
    BrainIO *brainio = new BrainIO();
    BrainRegistration *brainreg = new BrainRegistration();


    // Parameters
    method = atoi(argv[argn++]);
    fixedImageName = argv[argn++];
    ProbabilityImage fixed = brainio->ReadProbabilityImage(fixedImageName);
    movingImageName = argv[argn++];
    ProbabilityImage moving = brainio->ReadProbabilityImage(movingImageName);
    brainreg->SetFixed(fixed);
    brainreg->SetMoving(moving);

    brainreg->UseMattesMutualInformationMetric();

    // Registration
    switch (method) {
    case 1:

        std::cout<<"Rigid Registration: fixed image "<<argv[2]<<std::endl<<"moving image: "<<argv[3]<<std::endl; 	 

        levelsRigid = atoi(argv[argn++]);
        iterationsRigid = atoi(argv[argn++]);
        brainreg->RigidRegistration(levelsRigid,iterationsRigid);
        //sprintf(outputImageName,"%s_rigid.nii",movingImageName.substr(0,movingImageName.find_last_of('.')).c_str());

	sprintf(outputImageName,"outputRigid.nii");

        break;

    case 2:
        std::cout<<"Rigid plus Affine Registration: fixed image "<<argv[2]<<std::endl<<"moving image: "<<argv[3]<<std::endl; 	 

        levelsRigid = atoi(argv[argn++]);
        iterationsRigid = atoi(argv[argn++]);
        levelsAffine = atoi(argv[argn++]);
        iterationsAffine = atoi(argv[argn++]);
        brainreg->RigidRegistration(levelsRigid,iterationsRigid);
        brainreg->AffineRegistration(levelsAffine,iterationsAffine);
        //sprintf(outputImageName,"%s_affine.nii",movingImageName.substr(0,movingImageName.find_last_of('.')).c_str());
        sprintf(outputImageName,"outputRigAfi.nii");
        break;

    case 3:
        std::cout<<"Rigid plus Affine plus Bsplines Registration: fixed image "<<argv[2]<<std::endl<<"moving image: "<<argv[3]<<std::endl; 	 

        levelsRigid = atoi(argv[argn++]);
        iterationsRigid = atoi(argv[argn++]);
        levelsAffine = atoi(argv[argn++]);
        iterationsAffine = atoi(argv[argn++]);
        nodesBspline = atoi(argv[argn++]);
        levelsBspline = atoi(argv[argn++]);
        iterationsBspline = atoi(argv[argn++]);
        brainreg->RigidRegistration(levelsRigid,iterationsRigid);
        brainreg->AffineRegistration(levelsAffine,iterationsAffine);
        brainreg->BSplineMultiRegistration(levelsBspline,nodesBspline,iterationsBspline);
        //sprintf(outputImageName,"%s_bsplines.nii",movingImageName.substr(0,movingImageName.find_last_of('.')).c_str());
	sprintf(outputImageName,"outputRigAfiBsp.nii");

        break;
    }

    brainio->WriteProbabilityImage(outputImageName,brainreg->GetMoving());


     return EXIT_SUCCESS;


}
